@@ -2,7 +2,8 @@
 % Some basic data and predicates that are used by diffent modules.
 
 :- module(basics,[exitCode/1, setExitCode/1, failWithExitCode/0,
-		  noLoadMessage/0, lastload/1, plprofiling/1,
+		  noLoadMessage/0, withColor/1, cpmVersion/1, lastload/1,
+                  plprofiling/1,
 		  setVerbosity/1, verbosityQuiet/0, verbosityNotQuiet/0,
 		  verbosityIntermediate/0, verbosityDetailed/0,
 		  verbosemode/1, setVerboseMode/1, quietmode/1, setQuietMode/1,
@@ -17,7 +18,7 @@
 		  writeBlanks/1,
 		  onlySICStusMessage/1, checkSICStusAndWarn/1,
 		  onlySWIMessage/1, checkSWIAndWarn/1,
-		  putChars/2, writeChars/2,
+		  putChars/1, putChars/2, writeChars/2,
 		  assertPakcsrc/1, writeRCvalues/0,
 		  evaluateDynamicPredInfo/3, checkDynamicAccessMethod/2,
 		  resetDynamicPreds/0, clearDynamicPreds/0,
@@ -34,6 +35,7 @@
 		  findPrologTargetFileInLoadPath/2, findFilePropertyInPath/4,
 		  toAbsPath/2, split2dirbase/3, stripSuffix/2,
 		  isIoType/1, isId/1, 
+		  versionAtom/2, padVersionAtom/2, padList/4,
 		  constructorOrFunctionType/4,
 		  flatName2Atom/2, decodePrologName/2,
 		  isTupleCons/1, isLetterDigitCode/1,
@@ -48,12 +50,14 @@
 		  retractAllFacts/1, prefixComma/3,
 		  isWritableFile/1, tryWriteFile/1,
                   tryDeleteFile/1, deleteFileIfExists/1,
-		  ensureDirOfFile/1, prog2DirProg/2, prog2PrologFile/2,
+		  ensureDirOfFile/1, getOutDirectory/1, prog2DirProg/2, prog2PrologFile/2,
 		  prog2InterfaceFile/2, prog2FlatCurryFile/2,
 		  prog2ICurryFile/2, hierarchical2dirs/2,
-		  readStreamLine/2, removeBlanks/2, skipblanks/2,
+		  skipEOL/0, readStreamLine/2, removeBlanks/2, skipblanks/2,
 		  numberconst/3, readFileContents/2, readStreamContents/2,
-		  printError/1, prologError2Atom/2]).
+		  printError/1, prologError2Atom/2,
+                  readChar/3, readString/3,
+                  showTermChar/3, showTermString/3]).
 
 :- use_module(prologbasics).
 :- use_module(pakcsversion).
@@ -62,8 +66,8 @@
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-:- dynamic lastload/1, plprofiling/1, quietmode/1, verbosemode/1, rtArgs/1,
-	   compileWithSharing/1,
+:- dynamic withColor/1, cpmVersion/1, lastload/1, plprofiling/1, quietmode/1,
+	   verbosemode/1, rtArgs/1, compileWithSharing/1,
 	   compileWithDebug/0, compileWithFailPrint/0, hasPrintedFailure/0,
 	   printConsFailure/1, exitCode/1,
 	   user:dynamicPredInfo/2, orgDynamicPredInfo/2.
@@ -97,6 +101,8 @@ user:portray_message(informational,created(_,_)) :- !.
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
+withColor(yes).      % use colors for output
+cpmVersion('').      % the version string of CPM
 lastload("Prelude"). % program in last load command
 quietmode(no).	% yes if environment should work in quiet mode (option -q)
 plprofiling(no). % perform profiling with Prolog profiler
@@ -170,6 +176,10 @@ writeLnErrNQ(T) :- writeErrNQ(T), nlErrNQ.
 writeBlanks(N) :- N>0 -> put_code(32), N1 is N-1, writeBlanks(N1)
 	               ; true.
 
+% write a Prolog string (list of ASCII values):
+putChars([]).
+putChars([C|Cs]) :- put_code(C), putChars(Cs).
+
 % write a Prolog string (list of ASCII values) to a stream (Arg 1):
 putChars(_,[]).
 putChars(Stream,[C|Cs]) :-
@@ -632,6 +642,25 @@ prefixComma(C1,N,C2) :- N>1, N1 is N-1, prefixComma([44|C1],N1,C2).
 isId(T) :- atom_codes(T,[F|_]),
 	(65=<F, F=<90 ; 97=<F, F=<122).
 
+% convert the given version to an atom
+versionAtom(Version,Atom) :-
+	number_chars(Version,Chars),
+	atom_chars(Atom,Chars).
+
+% convert the given version to a zero-padded atom
+padVersionAtom(Version,PaddedAtom) :-
+	number_chars(Version,Chars),
+	padList(Chars,'0',2,PaddedChars),
+	atom_chars(PaddedAtom,PaddedChars).
+
+% pads a list with the given element
+padList(List,_,Length,PaddedList) :-
+	length(List,Length), !,
+	PaddedList = List.
+padList(List,Pad,Length,PaddedList) :-
+	length(List, Length2), Length2 < Length,
+	padList([Pad|List],Pad,Length,PaddedList).
+
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 % Axuiliaries for handling translated Prolog files
 
@@ -685,6 +714,17 @@ makeDirectoryWithPrefix(PrefixS,DirS) :-
 	atom_codes(CompleteDir,CompleteDirS),
 	(existsDirectory(CompleteDir) -> true ; makeDirectory(CompleteDir)).
 
+% fetch the name of the directory for compilation artifacts, e.g.
+% FlatCurry and compiled Prolog files.
+getOutDirectory(OutDir) :-
+	compilerMajorVersion(MajorVersion),
+	compilerMinorVersion(MinorVersion),
+	compilerRevisionVersion(RevVersion),
+	versionAtom(MajorVersion,MajorVersionAtom),
+	versionAtom(MinorVersion,MinorVersionAtom),
+	versionAtom(RevVersion,RevVersionAtom),
+	appendAtoms(['.curry/pakcs-',MajorVersionAtom,'.',MinorVersionAtom,'.',RevVersionAtom], OutDir).
+
 % generate the name of the program file for a given Curry program name,
 % i.e., replace all dots in a module name by slashes:
 prog2DirProg(Prog,ProgDir) :-
@@ -698,25 +738,29 @@ prog2DirProg(Prog,ProgDir) :-
 prog2PrologFile(Prog,PrologFile) :-
 	split2dirbase(Prog,ProgDir,ProgBase),
 	hierarchical2dirs(ProgBase,DProgBase),
-	appendAtoms([ProgDir,'/.curry/pakcs/',DProgBase,'.pl'],PrologFile).
+	getOutDirectory(OutDir),
+	appendAtoms([ProgDir,'/',OutDir,'/',DProgBase,'.pl'],PrologFile).
 
 % generate the name of the interface file for a given Curry program name:
 prog2InterfaceFile(Prog,IntFile) :-
 	split2dirbase(Prog,ProgDir,ProgBase),
 	hierarchical2dirs(ProgBase,DProgBase),
-	appendAtoms([ProgDir,'/.curry/',DProgBase,'.fint'],IntFile).
+	getOutDirectory(OutDir),
+	appendAtoms([ProgDir,'/',OutDir,'/',DProgBase,'.fint'],IntFile).
 
 % generate the name of the FlatCurry file for a given Curry program name:
 prog2FlatCurryFile(Prog,FlatFile) :-
 	split2dirbase(Prog,ProgDir,ProgBase),
 	hierarchical2dirs(ProgBase,DProgBase),
-	appendAtoms([ProgDir,'/.curry/',DProgBase,'.fcy'],FlatFile).
+	getOutDirectory(OutDir),
+	appendAtoms([ProgDir,'/',OutDir,'/',DProgBase,'.fcy'],FlatFile).
 
 % generate the name of the InterfaceCurry file for a given Curry program name:
 prog2ICurryFile(Prog,ICurryFile) :-
 	split2dirbase(Prog,ProgDir,ProgBase),
 	hierarchical2dirs(ProgBase,DProgBase),
-	appendAtoms([ProgDir,'/.curry/',DProgBase,'.icurry'],ICurryFile).
+	getOutDirectory(OutDir),
+	appendAtoms([ProgDir,'/',OutDir,'/',DProgBase,'.icurry'],ICurryFile).
 
 % translate a hierarchical module name (atom) into directory/file name,
 % i.e., replace dots by slashes:
@@ -726,6 +770,15 @@ hierarchical2dirs(HModName,DModName) :-
 	atom_codes(DModName,DModNameS).
 dot2slash(C1,C2) :- C1=46 -> C2=47 ; C2=C1.
 
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+% read input until newline or end-of-input is encountered.
+skipEOL :- skipTo(10).
+
+% read input until a code or end-of-input is encountered.
+skipTo(Code) :-
+	get_code(C),
+        ((C = -1 ; C = Code) -> true ; skiptTo(Code)).
+
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 % read a line from a stream:
 readStreamLine(Str,[]) :-
@@ -1060,3 +1113,134 @@ prologTerm2Atom(T,A) :-
 prefixComma(Atom,CAtom) :- appendAtom(',',Atom,CAtom).
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+% Definition of reading and showing single characters and strings.
+% This is contained here since it is used by the primitives of the prelude
+% and the library ReadShowTerm.
+
+
+readChar([92,N|S],T,C) :- N>=48, N<58, !, % read decimal numeric char
+	readDecimalChar(0,[N|S],T,C).
+readChar([92,N,39|T],T,C) :- !,
+	readStringChar(N,NS),
+	char_int(C,NS).
+readChar([92,78,85,76,39|T],T,C) :- !, char_int(C,0). % '\NUL' character
+readChar([92,83,79,72,39|T],T,C) :- !, char_int(C,1). % '\SOH' character
+readChar([92,83,84,88,39|T],T,C) :- !, char_int(C,2). % '\STX' character
+readChar([92,69,84,88,39|T],T,C) :- !, char_int(C,3). % '\ETX' character
+readChar([92,69,79,84,39|T],T,C) :- !, char_int(C,4). % '\EOT' character
+readChar([92,69,78,81,39|T],T,C) :- !, char_int(C,5). % '\ENQ' character
+readChar([92,65,67,75,39|T],T,C) :- !, char_int(C,6). % '\ACK' character
+readChar([92,97,39|T],T,C) :- !, char_int(C,7). % '\a' character
+readChar([92,66,69,76,39|T],T,C) :- !, char_int(C,7). % '\BEL' character
+readChar([92,98,39|T],T,C) :- !, char_int(C,8). % '\b' character
+readChar([92,66,83,39|T],T,C) :- !, char_int(C,8). % '\BS' character
+readChar([92,116,39|T],T,C) :- !, char_int(C,9). % '\t' character
+readChar([92,72,84,39|T],T,C) :- !, char_int(C,9). % '\HT' character
+readChar([92,110,39|T],T,C) :- !, char_int(C,10). % '\n' character
+readChar([92,76,70,39|T],T,C) :- !, char_int(C,10). % '\LF' character
+readChar([92,118,39|T],T,C) :- !, char_int(C,11). % '\v' character
+readChar([92,86,84,39|T],T,C) :- !, char_int(C,11). % '\VT' character
+readChar([92,102,39|T],T,C) :- !, char_int(C,12). % '\f' character
+readChar([92,70,70,39|T],T,C) :- !, char_int(C,12). % '\FF' character
+readChar([92,114,39|T],T,C) :- !, char_int(C,13). % '\r' character
+readChar([92,67,82,39|T],T,C) :- !, char_int(C,13). % '\CR' character
+readChar([92,83,79,39|T],T,C) :- !, char_int(C,14). % '\SO' character
+readChar([92,83,73,39|T],T,C) :- !, char_int(C,15). % '\SI' character
+readChar([92,68,76,69,39|T],T,C) :- !, char_int(C,16). % '\DLE' character
+readChar([92,68,67,49,39|T],T,C) :- !, char_int(C,17). % '\DC1' character
+readChar([92,68,67,50,39|T],T,C) :- !, char_int(C,18). % '\DC2' character
+readChar([92,68,67,51,39|T],T,C) :- !, char_int(C,19). % '\DC3' character
+readChar([92,68,67,52,39|T],T,C) :- !, char_int(C,20). % '\DC4' character
+readChar([92,78,65,75,39|T],T,C) :- !, char_int(C,21). % '\NAK' character
+readChar([92,83,89,78,39|T],T,C) :- !, char_int(C,22). % '\SYN' character
+readChar([92,69,84,66,39|T],T,C) :- !, char_int(C,23). % '\ETB' character
+readChar([92,67,65,78,39|T],T,C) :- !, char_int(C,24). % '\CAN' character
+readChar([92,69,77,39|T],T,C) :- !, char_int(C,25). % '\EM' character
+readChar([92,83,85,66,39|T],T,C) :- !, char_int(C,26). % '\SUB' character
+readChar([92,69,83,67,39|T],T,C) :- !, char_int(C,27). % '\ESC' character
+readChar([92,70,83,39|T],T,C) :- !, char_int(C,28). % '\FS' character
+readChar([92,71,83,39|T],T,C) :- !, char_int(C,29). % '\GS' character
+readChar([92,82,83,39|T],T,C) :- !, char_int(C,30). % '\RS' character
+readChar([92,85,83,39|T],T,C) :- !, char_int(C,31). % '\US' character
+readChar([92,83,80,39|T],T,C) :- !, char_int(C,32). % '\SP' character
+readChar([92,68,69,76,39|T],T,C) :- !, char_int(C,127). % '\DEL' character
+readChar([N,39|T],T,C) :- char_int(C,N).
+
+readDecimalChar(N,[39|T],T,C) :- !, char_int(C,N).
+readDecimalChar(N,[M|S],T,C) :- M>=48, M<58, !,
+	NM is 10*N+M-48,
+	readDecimalChar(NM,S,T,C).
+
+readString([34|T],T,[]) :- !.
+readString([92|Ns],T,Str) :- !, readStringEscape(Ns,T,Str).
+readString([N|Ns],T,[C|Str]) :-
+	char_int(C,N),
+	readString(Ns,T,Str).
+
+readStringEscape([N|Ns],T,Str) :- N>=48, N<58, !,
+        V is N-48, readDecimalCharInString(V,Ns,T,Str).
+readStringEscape([69,83,67|Ns],T,[C|Str]) :- !, % '\ESC' character
+        char_int(C,27),
+        readString(Ns,T,Str).
+readStringEscape([68,69,76|Ns],T,[C|Str]) :- !, % '\DEL' character
+        char_int(C,127),
+        readString(Ns,T,Str).
+readStringEscape([N|Ns],T,[C|Str]) :- !,
+	readStringChar(N,NS),
+	char_int(C,NS),
+	readString(Ns,T,Str).
+
+% read a character with decimal number representation, e.g., '\243'
+readDecimalCharInString(V,[N|Ns],T,Str) :- N>=48, N<58, !,
+        V1 is V*10+N-48,
+        readDecimalCharInString(V1,Ns,T,Str).
+readDecimalCharInString(V,Ns,T,[C|Str]) :-
+        char_int(C,V),
+        readString(Ns,T,Str).
+
+readStringChar(97,7) :- !.
+readStringChar(98,8) :- !.
+readStringChar(116,9) :- !.
+readStringChar(110,10) :- !.
+readStringChar(118,11) :- !.
+readStringChar(102,12) :- !.
+readStringChar(114,13) :- !.
+readStringChar(34,34) :- !.
+readStringChar(39,39) :- !.
+readStringChar(92,92) :- !.
+readStringChar(N,N) :-
+	writeErr('INTERNAL ERROR: unknown character string "'),
+	put_code(user_error,92), put_code(user_error,N),
+	writeErr('" in readStringChar'), nlErr.
+
+% The show functions use difference lists.
+
+showTermChar( 7,[C1,C2|E],E) :- !, cp_string([C1,C2],[92,97]).  % \a
+showTermChar( 8,[C1,C2|E],E) :- !, cp_string([C1,C2],[92,98]).  % \b
+showTermChar( 9,[C1,C2|E],E) :- !, cp_string([C1,C2],[92,116]). % \t
+showTermChar(10,[C1,C2|E],E) :- !, cp_string([C1,C2],[92,110]). % \n
+showTermChar(11,[C1,C2|E],E) :- !, cp_string([C1,C2],[92,118]). % \v
+showTermChar(12,[C1,C2|E],E) :- !, cp_string([C1,C2],[92,102]). % \f
+showTermChar(13,[C1,C2|E],E) :- !, cp_string([C1,C2],[92,114]). % \r
+showTermChar(34,[C1,C2|E],E) :- !, cp_string([C1,C2],[92,34]). % 34="
+showTermChar(92,[C1,C2|E],E) :- !, cp_string([C1,C2],[92,92]). % 92=\
+showTermChar(N,[C1,C2,C3|E],E) :- N<32, !,
+	N1 is (N//10)+48, N2 is (N mod 10)+48,
+	cp_string([C1,C2,C3],[92,N1,N2]).
+showTermChar(N,E,F) :- N>126, !,
+	num2rdigits(N,RDs), rev(RDs,Ds),
+	cp_string(DS,[92|Ds]),
+	append(DS,F,E).
+showTermChar(N,[C|E],E) :- char_int(C,N).
+
+num2rdigits(N,[D]) :- N<10, !, D is N+48.
+num2rdigits(N,[D|Ds]) :- D is (N mod 10)+48, N1 is N//10, num2rdigits(N1,Ds).
+
+
+showTermString([],[Quot|E],E) :- char_int(Quot,34).
+showTermString([C|T],S,E) :-
+	char_int(C,N),
+	showTermChar(N,S,ST),
+	showTermString(T,ST,E).
+
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
