@@ -16,15 +16,13 @@
 
 :- dynamic compileWithCompact/1,
 	   parser_warnings/1, parserOptions/1,
-	   freeVarsUndeclared/1, varDefines/1,
-	   addImports/1, safeMode/1.
+	   freeVarsUndeclared/1, addImports/1, safeMode/1.
 
 compileWithCompact([]).  % parsecurry options for compactification
 parser_warnings(yes). % no if the warnings of the parser should be suppressed
 parserOptions(''). % additional options passed to front end
 freeVarsUndeclared(no). % yes if free variables need not be declared in initial goals
 addImports([]). % additional imports defined by the ":add" command
-varDefines([]). % list of top-level bindings
 safeMode(no). % safe execution without IO actions at top level?
 
 % Read the PAKCS rc file to define some constants
@@ -36,11 +34,7 @@ readRcFile(ArgProps) :-
 	% first, try to install local .pakcsrc file:
         (existsFile(HomeConfigFile)
 	 -> readConfigFile(HomeConfigFile,HomeProps)
-	  ; appendAtoms(['cp ',ConfigFile,' ',HomeConfigFile],CpCmd),
-	    shellCmd(CpCmd),
-	    writeNQ('>>> '),
-	    writeNQ(HomeConfigFile), writeNQ(' installed.'), nlNQ,
-	    HomeProps=[] ),
+	  ; HomeProps=[] ),
 	(existsFile(ConfigFile)
 	 -> readConfigFile(ConfigFile,GlobalProps),
 	    updateConfigFile(ConfigFile,HomeProps,HomeConfigFile)
@@ -71,6 +65,21 @@ readRcFile(ArgProps) :-
 	map1M(basics:assertPakcsrc,UniqueProps).
 
 
+% Try to install the PAKCS rc file in home dir if not already there:
+installRcFileIfNotPresent :-
+	installDir(PH),
+        appendAtom(PH,'/pakcsrc.default',ConfigFile),
+        getHomeDirectory(Home),
+	appendAtom(Home,'/.pakcsrc',HomeConfigFile),
+	!,
+        (existsFile(HomeConfigFile)
+	 -> true
+	  ; appendAtoms(['cp ',ConfigFile,' ',HomeConfigFile],CpCmd),
+	    shellCmd(CpCmd),
+	    writeNQ('>>> '),
+	    writeNQ(HomeConfigFile), writeNQ(' installed.'), nlNQ).
+installRcFileIfNotPresent.
+
 deletePropDups([],[]).
 deletePropDups([prop(Name,Value)|Props],[prop(Name,Value)|DProps]) :-
 	deleteEqualProps(Name,Props,Props1),
@@ -102,8 +111,9 @@ pakcsMain :-
 	((RTArgs=[], \+ verbosityIntermediate) -> true
 	  ; writeNQ('Run-time parameters passed to application: '),
 	    writeNQ(RTArgs), nlNQ),
+        installRcFileIfNotPresent,
 	(verbosityNotQuiet
-         -> printPakcsHeader, nlNQ,
+         -> printPakcsHeader,
             writeNQ('Type ":h" for help (contact: pakcs@curry-lang.org)'),
             nlNQ
           ; true),
@@ -120,14 +130,27 @@ processDArgs([Arg|DArgs],[prop(Name,Val)|Props],Args) :-
 	atom_codes(Name,NameS),
 	atom_codes(Val,ValS),
 	processDArgs(DArgs,Props,Args).
+processDArgs([Arg|Args],[],[Arg|Args]) :-
+	atom_codes(Arg,[58|_]), !. % 58 = ':', REPL command, stop processing
+processDArgs([Arg|DArgs],Props,[Arg|Args]) :-
+	!, % store other args
+	processDArgs(DArgs,Props,Args).
 processDArgs(Args,[],Args).
 
 % process the remaining run-time arguments:
 processArgs(Halt,[]) :- Halt=yes -> halt(0) ; true.
+processArgs(Halt,['--cpm-version'|CArgs]) :-
+        (CArgs = [CV|Args] -> retract(cpmVersion(_)), asserta(cpmVersion(CV))
+                            ; Args = CArgs), % this case should not occur...
+        !,
+	processArgs(Halt,Args).
 processArgs(Halt,['--nocypm'|Args]) :-
 	processArgs(Halt,Args).            % ignore since already processed
 processArgs(Halt,['--noreadline'|Args]) :-
 	processArgs(Halt,Args).            % ignore since already processed
+processArgs(Halt,['--nocolor'|Args]) :-
+        retract(withColor(_)), asserta(withColor(no)), !,
+	processArgs(Halt,Args).
 processArgs(_,[Arg|Args]) :-
 	(Arg='--version' ; Arg='-V'), !, % show version and quit:
 	printPakcsHeader,
@@ -221,6 +244,7 @@ writeMainHelp :-
 	writeLnErr('-q|--quiet        : work silently'),
 	writeLnErr('--nocypm          : do not invoke "cypm" to compute package load path'),
 	writeLnErr('--noreadline      : do not use input line editing via command "rlwrap"'),
+	writeLnErr('--nocolor         : do not use colored output'),
 	writeLnErr('-Dprop=val        : define pakcsrc property "prop" as "val"'),
 	writeLnErr(':<cmd> <args>     : command of the PAKCS environment'),
 	nlErr,
@@ -231,12 +255,8 @@ writeMainHelp :-
 	nlErr,
 	writeLnErr('where <tool> is one of:'),
 	nlErr,
-	writeLnErr('browse    : browse and analyze'),
-	writeLnErr('check     : check properties'),
-	writeLnErr('doc       : generate documentation for Curry programs'),
-	writeLnErr('frontend  : Curry front end'),
-	writeLnErr('makecgi   : translate Curry HTML program into CGI program'),
-	writeLnErr('pp        : Curry preprocessor'),
+	writeLnErr('cypm     : Curry package manager'),
+	writeLnErr('frontend : Curry front end'),
         nlErr,
         writeLnErr('To get more help about the usage of a tool, type'),
 	nlErr,
@@ -432,6 +452,11 @@ flatType2Atom(Type,CurryType) :-
 % argument and the qualified class name (string) in the third (module)
 % and fourth (class name) argument:
 classDict(T,_,_,_) :- var(T), !, fail.
+% if type classes are translated as functions to implement run-time choice,
+% see https://git.ps.informatik.uni-kiel.de/curry/curry-frontend/-/issues/109
+classDict('FuncType'('TCons'('Prelude.()',[]),
+                     'TCons'(FCDict,[A])),A,ModS,DictS) :- !,
+        classDict('TCons'(FCDict,[A]),A,ModS,DictS).
 classDict('TCons'(FCDict,[A]),A,ModS,DictS) :-
         atomic2Codes(FCDict,FCDictS), % hack for SWI 7.x if FCDict == []
         % dictionary argument types are prefixed by "_Dict#":
@@ -476,7 +501,7 @@ parseExpressionWithFrontend(MainExprDir,Input,InitMainFuncType,MainExp,
 	loadPath(AbsMainPath,LoadPath),
 	setCurryPath(NewLCP),
 	setWorkingDirectory(MainExprDir),
-	readProg(['.'|LoadPath],'PAKCS_Main_Exp',FlatProg,_),
+	readProg(['.'|LoadPath],'PAKCS_Main_Exp',FlatProg,_,_),
 	setCurryPath(LCP), % restore old settings
 	setWorkingDirectory(CurDir),
 	FlatProg = 'Prog'(_,_Imps,_TDecls,FDecls,_),
@@ -581,7 +606,7 @@ removeDefaultedTypes(Type,Type) :- var(Type), !.
 removeDefaultedTypes('FuncType'(AType,RType),DType) :-
 	classDict(AType,TVar,ModName,DictName),
         ModName="Prelude", nonvar(TVar),
-	( member(DictName,["Eq","Ord","Read","Show"])
+	( member(DictName,["Data","Eq","Ord","Read","Show"])
         ; DictName="Enum",
           member(TVar,['TCons'('Prelude.Int',[]),'TCons'('Prelude.Float',[])])
         ), !,
@@ -712,17 +737,9 @@ writeMainExprFile(ExprFile,MainProg,Input,FreeVars,InitMainFuncType) :-
 	write(S,'pakcsMainGoal'),
 	writeFreeVarArgs(S,FreeVars),
 	write(S,' = '),
-	varDefines(VarDefs), writeVarDefs(S,VarDefs),
 	putChars(S,Input), nl(S),
 	close(S).
 
-writeVarDefs(_,[]).
-writeVarDefs(S,[Var=Exp|VarDefs]) :-
-	exp2Term(Exp,[],Term,_),
-	write(S,'let '), write(S,Var), write(S,' = '),
-	writeCurryOnStream(S,Term), write(S,' in '),
-	writeVarDefs(S,VarDefs).
-
 writeFreeVarArgs(_,[]).
 writeFreeVarArgs(S,[V|Vs]) :-
         write(S,' '), write(S,V), writeFreeVarArgs(S,Vs).
@@ -732,6 +749,11 @@ writeMainImports(S,[Imp|Imps]) :-
 	write(S,'import '), write(S,Imp), nl(S),
 	writeMainImports(S,Imps).
 
+% convert variables into Prolog variables:
+addVar(V,[],NV,[(V=NV)]).
+addVar(V,[(X=NX)|Vs],NX,[(X=NX)|Vs]) :- V==X, !.
+addVar(V,[(X=NX)|Vs],NV,[(X=NX)|Vs1]) :- addVar(V,Vs,NV,Vs1).
+
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
@@ -905,7 +927,6 @@ processCommand("reload",[]) :- !,
 	              asserta(currentprogram(Prog)),
 		      initializationsInProg(ProgInits), call(ProgInits)),
 	             printError(ErrorMsg) ),
-	retract(varDefines(_)), asserta(varDefines([])),
 	(compileWithDebug -> retract(spypoints(_)),
 	                     asserta(spypoints([])),
 	                     singleOn, traceOn, spyOff
@@ -1044,8 +1065,10 @@ processCommand("show",ShTail) :- % show source of a module
 	appendAtoms([Pager,' "',File,'"'],Cmd),
 	shellCmdWithReport(Cmd).
 
-processCommand("show",_) :- !,
-	writeLnErr('ERROR: Source file not found').
+processCommand("show",ShTail) :- !,
+	writeErr('ERROR: Source file "'),
+        atom_codes(ShTailA,ShTail), writeErr(ShTailA),
+        writeLnErr('" not found!').
 
 processCommand("source",Arg) :-
 	append(PModS,[46|FunS],Arg),
@@ -1539,8 +1562,12 @@ parseProgram(ProgS,Verbosity,Warnings) :-
 	(MinorVersion < 100 -> true
 	  ; writeLnErr('ERROR minor version too large!'), fail),
 	padVersionAtom(MinorVersion, PaddedMinorVersionAtom),
-	appendAtoms(['"',TCP,'/bin/pakcs-frontend" --flat -D__PAKCS__=',
-	  MajorVersionAtom,PaddedMinorVersionAtom],CM1),
+	getOutDirectory(OutDir),
+	appendAtoms(['"',TCP,'/bin/pakcs-frontend" --flat',
+                     ' -Odesugar-newtypes ',
+                     ' -o ', OutDir,
+                     ' -D__PAKCS__=',
+                     MajorVersionAtom,PaddedMinorVersionAtom],CM1),
 	(Warnings=no -> appendAtom(CM1,' -W none',CM2)    ; CM2 = CM1 ),
 	(Verbosity=0 -> appendAtom(CM2,' --no-verb',CM3)  ; CM3 = CM2 ),
 	((Warnings=yes, pakcsrc(warnoverlapping,no))
@@ -1581,22 +1608,6 @@ addImports([I|Is],CY1,CY3) :-
 	appendAtoms([CY1,' -i"',I,'"'],CY2),
 	addImports(Is,CY2,CY3).
 
-versionAtom(Version,Atom) :-
-	number_chars(Version,Chars),
-	atom_chars(Atom,Chars).
-
-padVersionAtom(Version,PaddedAtom) :-
-	number_chars(Version,Chars),
-	padList(Chars,'0',2,PaddedChars),
-	atom_chars(PaddedAtom,PaddedChars).
-
-padList(List,_,Length,PaddedList) :-
-	length(List,Length), !,
-	PaddedList = List.
-padList(List,Pad,Length,PaddedList) :-
-	length(List, Length2), Length2 < Length,
-	padList([Pad|List],Pad,Length,PaddedList).
-
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 % small pretty printer for type expressions:
 
@@ -1808,23 +1819,23 @@ call_singlestepmenu(Skip) :-
 	% if input trace/notrace -> show menu again
         (member(C,[110,116]) -> call_singlestepmenu(Skip) ; true), !.
 
-call_debug_option(103,_) :- !, skip(10),   % g: turn off single step mode
+call_debug_option(103,_) :- !, skipEOL,   % g: turn off single step mode
         singleOff, write('Single step mode off.'), nl.
-call_debug_option(116,_) :- !, skip(10),   % t: turn on trace mode
+call_debug_option(116,_) :- !, skipEOL,   % t: turn on trace mode
         traceOn, write('Trace mode on.'), nl.
-call_debug_option(110,_) :- !, skip(10),   % n: turn off trace mode
+call_debug_option(110,_) :- !, skipEOL,   % n: turn off trace mode
         traceOff, write('Trace mode off.'), nl.
-call_debug_option(115,skip) :- !, skip(10),   % s: skip over function call
+call_debug_option(115,skip) :- !, skipEOL,   % s: skip over function call
         singleOff, traceOff.
-call_debug_option(108,skip) :- !, skip(10),   % l: jump to next spy point
+call_debug_option(108,skip) :- !, skipEOL,   % l: jump to next spy point
         singleOff, traceOff, spyOn.
-call_debug_option(97,_) :- !, skip(10),   % a: abort
+call_debug_option(97,_) :- !, skipEOL,   % a: abort
         raise_exception(debugger_abort). %abort.
-call_debug_option(101,eval) :- !, skip(10),   % e: evaluate to normal form
+call_debug_option(101,eval) :- !, skipEOL,   % e: evaluate to normal form
 	singleOff, traceOff.
 call_debug_option(10,_) :- !. % <return>: do the next single step
 call_debug_option(_,S) :- write('ERROR: wrong option!'), nl,
-	skip(10), call_singlestepmenu(S).
+	skipEOL, call_singlestepmenu(S).
 
 % single step menu for exit ports:
 exit_singlestepmenu :-
@@ -1834,19 +1845,19 @@ exit_singlestepmenu :-
 	% if input trace/notrace -> show menu again
         (member(C,[110,116]) -> exit_singlestepmenu ; true), !.
 
-exit_debug_option(103) :- !, skip(10),   % g: turn off single step mode
+exit_debug_option(103) :- !, skipEOL,   % g: turn off single step mode
         singleOff, write('Single step mode off.'), nl.
-exit_debug_option(116) :- !, skip(10),   % t: turn on trace mode
+exit_debug_option(116) :- !, skipEOL,   % t: turn on trace mode
         traceOn, write('Trace mode on.'), nl.
-exit_debug_option(110) :- !, skip(10),   % n: turn off trace mode
+exit_debug_option(110) :- !, skipEOL,   % n: turn off trace mode
         traceOff, write('Trace mode off.'), nl.
-exit_debug_option(108) :- !, skip(10),   % l: jump to next spy point
+exit_debug_option(108) :- !, skipEOL,   % l: jump to next spy point
         singleOff, traceOff, spyOn.
-exit_debug_option(97) :- !, skip(10),   % a: abort
+exit_debug_option(97) :- !, skipEOL,   % a: abort
         raise_exception(debugger_abort). %abort.
 exit_debug_option(10) :- !. % do the next single step
 exit_debug_option(_) :- write('ERROR: wrong option!'), nl,
-	skip(10), exit_singlestepmenu.
+	skipEOL, exit_singlestepmenu.
 
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -1886,47 +1897,6 @@ transDefinedFunc(ModF,ModFAtom) :-
 
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-% Parser for expressions:
-%
-% current restrictions:
-% - no lambda abstractions
-% - no case expressions
-% - no let expressions
-
-hasfixity('.',infixr(5)) :- !. % hack for list cons operator
-hasfixity('Prelude..',infixr(9)) :- !. % hack for function composition operator
-hasfixity(O,Fixity) :-
-	functiontype(O,_,_,_,OFix,_),
-	(OFix=nofix -> Fixity=infixl(9) % non declared opids have fixity "infixl 9"
-	             ; Fixity=OFix).
-
-% parser for (define) bindings:
-mainbinding(X,E) --> id(XS), skipblanks, "=", skipblanks,
-	             expr(E), {atom_codes(X,XS)}.
-
-
-
-min(X,Y,Z) :- X<Y -> Z=X ; Z=Y.
-
-append3(X,Y,Z,L) :- append(XY,Z,L), append(X,Y,XY).
-
-
-var2comb(var(F),comb(TF,[])) :- transDefinedFunc(F,TF), !.
-var2comb(var(V),var(UV)) :- !,
-	(atom_codes(V,[95|_])  % 95 = '_'
-	 -> V=UV
-	  ; appendAtom('_',V,UV)).
-var2comb(E,E) :- \+ E=opid(_).
-
-generateApply(T,[],T).
-generateApply(T,[X|Xs],E) :- generateApply('Prelude.apply'(T,X),Xs,E).
-		   
-getArityFromType(T,0) :- var(T), !.
-getArityFromType('FuncType'(_,T2),N) :-
-	getArityFromType(T2,N2), N is N2+1.
-getArityFromType('TCons'(TC,_),N) :- TC="IO" -> N=1 ; N=0.
-
-
 % extract the program name from a given input, i.e., remove all blanks,
 % replace leading "~" by current home directory,
 % delete possible suffix ".curry" or ".lcurry", and delete absolute
@@ -1965,117 +1935,6 @@ isValidModuleString([C|Cs]) :-
 	isValidModuleString(Cs).
 
 
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5
-% convert an expression (of FlatCurry) into a Prolog term and return
-% list of assignments (variable -> Prolog variable):
-
-exp2Term(var(V),Vs,V1,Vs1) :- !, addVar(V,Vs,V1,Vs1).
-exp2Term(int(I),Vs,I,Vs) :- !.
-exp2Term(float(I),Vs,I,Vs) :- !.
-exp2Term(char(I),Vs,I_Atom,Vs) :- !, char_int(I_Atom,I).
-exp2Term(comb(Name,Exprs),Vs,CombTerm,Vs1) :- !,
-	exp2Terms(Exprs,Vs,Terms,Vs1),
-	(constructorOrFunctionType(Name,_,Arity,_)
-	  -> true
-           ; write('ERROR: type of function "'), write(Name),
-             write('" is unknown'), nl, !, fail),
-	length(Terms,TArity),
-	Missing is Arity-TArity,
-	Term =.. [Name|Terms],
-	term2partcall(Term,Missing,TermOrPartCall),
-	(compileWithSharing(function)
-	 -> CombTerm = makeShare(TermOrPartCall,_) % TODO: improve: not necessary for constr
-	  ; CombTerm = TermOrPartCall).
-exp2Term('Prelude.apply'(F,X),Vs,Term,Vs2) :- !,
-	exp2Term(F,Vs,FT,Vs1), exp2Term(X,Vs1,XT,Vs2),
-	(compileWithSharing(function)
-	 -> Term = makeShare('Prelude.apply'(FT,XT),_)
-	  ; Term = 'Prelude.apply'(FT,XT)).
-exp2Term(_Expr,_,_,_) :- write('*** Syntax error'), nl,
-	!, fail.
-
-exp2Terms([],Vs,[],Vs).
-exp2Terms([E|Es],Vs,[T|Ts],Vs2) :-
-	exp2Term(E,Vs,T,Vs1),
-	exp2Terms(Es,Vs1,Ts,Vs2).
-
-% convert variables into Prolog variables:
-addVar(V,[],NV,[(V=NV)]).
-addVar(V,[(X=NX)|Vs],NX,[(X=NX)|Vs]) :- V==X, !.
-addVar(V,[(X=NX)|Vs],NV,[(X=NX)|Vs1]) :- addVar(V,Vs,NV,Vs1).
-
-
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-% type check an expression (of FlatCurry):
-
-typecheck(Exp,Type) :-	typecheck(Exp,[],Type,_).
-
-typecheck(var(V),Vs,Type,Vs1) :- !, addVarType(V,Vs,Type,Vs1).
-typecheck(int(_),Vs,'TCons'('Prelude.Int',[]),Vs) :- !.
-typecheck(float(_),Vs,'TCons'('Prelude.Float',[]),Vs) :- !.
-typecheck(char(_),Vs,'TCons'('Prelude.Char',[]),Vs) :- !.
-typecheck(comb(Name,[]),Vs,FType,Vs) :- !,
-	(constructorOrFunctionType(Name,_,_,FType)
-	  -> true
-           ; write('ERROR: type of function "'), write(Name),
-             write('" is unknown'), nl, !, fail).
-typecheck(comb(Name,Exprs),Vs,Type,Vs1) :-
-	comb2apply([comb(Name,[])|Exprs],ApplyExpr),
-	typecheck(ApplyExpr,Vs,Type,Vs1).
-typecheck('Prelude.apply'(F,X),Vs,Type,Vs2) :-
-	typecheck(F,Vs,TF,Vs1), !,
-	typecheck(X,Vs1,TX,Vs2), !,
-	(unify_type(TF,'FuncType'(TA,Type)) -> true ;
-          write('ERROR: Type error in application: '),
- 	  exp2Term('Prelude.apply'(F,X),[],'Prelude.apply'(FT,XT),_),
-	  writeCurry('Prelude.apply'(FT,XT)), nl,
-	  write('*** term           : '), writeCurry(FT), nl,
-	  numbersmallvars(97,_,TF),
-	  write('*** type           : '), writeType(TF), nl,
-	  write('*** is not of functional type'), nl,
-	  !, fail),
-	(unify_type(TA,TX) -> true ;
-          write('ERROR: Type error in application: '),
- 	  exp2Term('Prelude.apply'(F,X),[],Term,_),
-	  writeCurry(Term), nl,
-	  numbersmallvars(97,_,(TA,TX)),
-	  write('*** required argument type : '), writeType(TA), nl,
-	  write('*** does not match         : '), writeType(TX), nl,
-	  !, fail).
-
-
-% translate an expression list into a nested apply:
-comb2apply([Expr],Expr) :- !.
-comb2apply(Exprs,'Prelude.apply'(AE,E)) :-
-	append(Es,[E],Exprs),
-	comb2apply(Es,AE).
-
-
-% add type information for a variable:
-addVarType(V,[],Type,[type(V,Type)]). % new type for first variable occurrence
-addVarType(V,[type(X,TX)|VTs],TX,[type(X,TX)|VTs]) :- V==X, !.
-addVarType(V,[type(X,TX)|VTs],TV,[type(X,TX)|VTs1]) :-
-	addVarType(V,VTs,TV,VTs1).
-
-% unify types: instantiate occurrences of type variables by the other type:
-unify_type(T1,T2) :- var(T1), var(T2), !, T1=T2.
-unify_type(T1,T2) :- var(T1), !, tvar_occurs_not(T1,T2), T1=T2.
-unify_type(T1,T2) :- var(T2), !, tvar_occurs_not(T2,T1), T1=T2.
-unify_type('FuncType'(T1,T2),'FuncType'(T3,T4)) :-
-	unify_type(T1,T3), unify_type(T2,T4).
-unify_type('TCons'(TC,TArgs1),'TCons'(TC,TArgs2)) :-
-	unify_types(TArgs1,TArgs2).
-
-unify_types([],[]) :- !.
-unify_types([T1|Ts1],[T2|Ts2]) :- unify_type(T1,T2), unify_types(Ts1,Ts2).
-
-tvar_occurs_not(TV,T) :- var(T), !, TV\==T.
-tvar_occurs_not(TV,'FuncType'(T1,T2)) :-
-	tvar_occurs_not(TV,T1), tvar_occurs_not(TV,T2).
-tvar_occurs_not(TV,'TCons'(_,TArgs)) :-
-	map1partialM(user:tvar_occurs_not(TV),TArgs).
-
-
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 % Read a configuration file:
@@ -2108,6 +1967,8 @@ readStreamLines(Str,[Line|Lines]) :-
 
 
 % Update a configuration file with a list of given properties
+updateConfigFile(OrgFile,_,UpdFile) :-
+        \+ (existsFile(OrgFile), existsFile(UpdFile)), !. % nothing to do
 updateConfigFile(OrgFile,_,UpdFile) :-
 	fileModTime(OrgFile,OrgModTime),
 	fileModTime(UpdFile,UpdModTime),
@@ -2156,11 +2017,15 @@ showSourceCode(F) :- var(F),
 	writeLnErr('Cannot show source code of a variable!').
 showSourceCode(partcall(_,QF,_)) :- !,
 	atom_codes(QF,QFS),
-	append(ModS,[46|FS],QFS), !,
+	append(ModS,[46|FS],QFS),
+        (\+ member(46,FS) ; isOperatorName(FS)),
+        !,
 	showSourceCodeOfFunction(ModS,FS).
 showSourceCode(FCall) :- FCall =.. [QF|_],
 	atom_codes(QF,QFS),
-	append(ModS,[46|FS],QFS), !,
+	append(ModS,[46|FS],QFS),
+        (\+ member(46,FS) ; isOperatorName(FS)),
+        !,
 	showSourceCodeOfFunction(ModS,FS).
 
 % show source code of a function via the simple GUI for showing complete fun's:
@@ -2193,6 +2058,8 @@ getModStream(ModS,InStream) :-
         checkCpmTool('curry-showsource','sourceproggui',ShowSource),
 	atom_codes(ModA,ModS),
 	appendAtoms([ShowSource,' ',ModA,' 2>/dev/null'],Cmd),
+	(verbosityIntermediate -> write('Executing: '), write(Cmd), nl ; true),
+	flush_output(user_output),
    	execCommand(Cmd,InStream,_,std),
 	assertz(sourceCodeGUI(ModS,InStream)).
 
