@@ -16,7 +16,7 @@
 
 :- use_module(prologbasics).
 :- use_module(basics).
-:- ensure_lib_loaded(prim_readshowterm).
+:- use_module(readShowTerm).
 
 :- (swi7orHigher -> set_prolog_flag(double_quotes, codes) ; true).
 
@@ -280,7 +280,7 @@ failureInteraction(FTLen,FailSrc) :-
         failureIntOption(FTLen,FailSrc,C).
 
 failureIntOption(FTLen,FailSrc,104) :- !, % help
-	skip(10),
+	skipEOL,
 	write('Commands in interactive mode for failure tracing:'), nl,
 	write('l     - list complete trace from root to failed call'), nl,
 	write('l <n> - show last <n> elements of trace'), nl,
@@ -309,14 +309,14 @@ failureIntOption(FTLen,FailSrc,115) :- !, % show
 	writeCurry(FCall), nl,
 	!, failureInteraction(FTLen,FailSrc).
 failureIntOption(FTLen,FailSrc,102) :- !, % functions
-	skip(10),
+	skipEOL,
 	writeFunctionFailureList(user_output,FTLen,FailSrc),
 	failureInteraction(FTLen,FailSrc).
 failureIntOption(FTLen,FailSrc,112) :- !, % printdepth
 	readLine(PDLine),
 	removeBlanks(PDLine,PDL),
 	(codes2number(PDL,D)
-	 -> retract(printDepth(_)),	
+	 -> retract(printDepth(_)),
 	    (D=0 -> D1=D ; D1 is D+1),
 	    asserta(printDepth(D1))
 	  ; write('Illegal print depth'), nl),
@@ -325,12 +325,12 @@ failureIntOption(FTLen,FailSrc,112) :- !, % printdepth
 	nl,
 	!, failureInteraction(FTLen,FailSrc).
 failureIntOption(_,_,113) :- !, % quit
-	skip(10), fail.
+	skipEOL, fail.
 failureIntOption(FTLen,FailSrc,C) :-
 	write('ERROR: wrong option!'), nl,
-	(C=10 -> true ; skip(10)),
+	(C=10 -> true ; skipEOL),
 	failureInteraction(FTLen,FailSrc).
-	
+
 writeAllFailureList(Stream,_,[FName,Args]) :- !,
 	writeFailedCall(Stream,FName,Args).
 writeAllFailureList(Stream,FTLen,[FCall|FailSrc]) :- !,
@@ -341,6 +341,8 @@ writeAllFailureList(Stream,FTLen,[FCall|FailSrc]) :- !,
 
 writeFunctionFailureList(Stream,_,[partcall(_,FName,_),Args]) :- !,
 	writeFailedCall(Stream,FName,Args).
+writeFunctionFailureList(Stream,_,[FName,Args]) :- !,
+	writeFailedCall(Stream,FName,Args).
 writeFunctionFailureList(Stream,FTLen,[FCall|FailSrc]) :- !,
 	FCall =.. [Fun|_],
 	writeFailCallNumber(Stream,FTLen),
@@ -371,7 +373,7 @@ tryWriteSuspGoal(_:G) :-
 	write('let '), writeCurry(Result), write(' = '),
 	writeCurry(FunCall), nl,
 	!.
-tryWriteSuspGoal(_:G) :- !, 
+tryWriteSuspGoal(_:G) :- !,
 	writeCurry(G), nl.
 tryWriteSuspGoal(G) :- write(G), nl.
 
@@ -476,7 +478,7 @@ writeCurryD(S,D,_,T) :-
 	!,
 	(isString(TL)
 	    -> % use ReadShowTerm.showTerm for showing strings:
-	       user:show_term(TL,_,TS,[]), string2Atom(TS,TA), write(S,TA)
+	       show_term(TL,_,TS,[]), string2Atom(TS,TA), write(S,TA)
 	     ; write(S,'['), writeCurryList(S,D,TL)).
 writeCurryD(S,D,Nested,[T|Ts]) :- !,
 	(Nested=nested -> write(S,'(') ; true),
@@ -490,7 +492,7 @@ writeCurryD(S,D,Nested,'Prelude.apply'(F,X)) :- !,
 	writeCurryD(S,D1,nested,F), write(S,' '),
 	writeCurryD(S,D1,nested,X),
 	(Nested=nested -> write(S,')') ; true).
-writeCurryD(S,D,Nested,'Prelude.if_then_else'(C,T,E)) :- !,
+writeCurryD(S,D,Nested,'Prelude.ifThenElse'(C,T,E)) :- !,
 	D1 is D-1,
 	(Nested=nested -> write(S,'(') ; true),
 	write(S,'if '), writeCurryD(S,D1,top,C),
@@ -507,7 +509,8 @@ writeCurryD(S,D,_,T) :-
 	write(S,')').
 writeCurryD(S,D,Nested,T) :-
 	D1 is D-1,
-	T =.. [IntCons,Arg1,Arg2],
+	T =.. [IntCons|Args],
+        omitClassDicts(Args,[Arg1,Arg2]),
 	revTransFunctor(IntCons,Cons),
 	\+ isId(Cons),		% write as an infix operator:
 	!,
@@ -521,19 +524,24 @@ writeCurryD(S,D,Nested,T) :-
 	T =.. [IntCons|Args],
 	revTransFunctor(IntCons,Cons),
 	(Nested=nested -> write(S,'(') ; true),
-	write(S,Cons),
-	writeCurryArgs(S,D1,Nested,Args).
+	(isId(Cons) -> write(S,Cons)
+                     ; write(S,'('), write(S,Cons), write(S,')')),
+        omitClassDicts(Args,RArgs),
+	writeCurryArgs(S,D1,Nested,RArgs).
+
+% omit class dictionary arguments
+omitClassDicts([],[]).
+omitClassDicts([A|As],Bs) :- isInstDict(A), !, omitClassDicts(As,Bs).
+omitClassDicts([A|As],[A|Bs]) :- omitClassDicts(As,Bs).
 
 writeCurryArgs(S,_,Nested,[]) :- (Nested=nested -> write(S,')') ; true).
-writeCurryArgs(S,D,Nested,[A|As]) :- isInstDict(A), !, % omit class dicts
-        writeCurryArgs(S,D,Nested,As).
 writeCurryArgs(S,D,Nested,[A|As]) :-
 	write(S,' '),
 	writeCurryD(S,D,nested,A),
 	writeCurryArgs(S,D,Nested,As).
 
 writeCurryLiteral(S,L) :- % use ReadShowTerm.showTerm for showing number/char literals
-	user:show_term(L,_,CS,[]),
+	show_term(L,_,CS,[]),
 	string2Atom(CS,CA),
 	write(S,CA).
 
@@ -568,7 +576,7 @@ revTransFunctor(IntName,Name) :-
 revTransFunctor(Name,Name).
 
 % is this a class dictionary?
-isInstDict(T) :- var(T), fail.
+isInstDict(T) :- var(T), !, fail.
 isInstDict(T) :- atom(T), !, isInstDictName(T).
 isInstDict(T) :- number(T), !, fail.
 isInstDict(T) :- T =.. [Name|_], isInstDictName(Name).
