@@ -19,7 +19,7 @@
 	   waitConcurrentConjunction/6,
 	   appendAtom/3,
 	   map1M/2, map2M/3, map1partialM/2, map2partialM/3,
-	   getProgramArgs/1, getEnv/2,
+	   getProgramArgs/1, getEnv/2, setEnv/2, unsetEnv/1,
 	   noSingletonWarnings/0, noRedefineWarnings/0, noDiscontiguousWarnings/0,
 	   getRunTime/1, getElapsedTime/1,
 	   getCurrentMemorySize/1, getCurrentCodeSize/1,
@@ -320,6 +320,12 @@ getProgramArgs(Args) :- prolog_flag(argv,Args,Args).
 % get value of environment variable (fails if it is not set):
 getEnv(Var,Val) :- environ(Var,Val), !.
 
+% set an environment variable to some value:
+setEnv(Var,Val) :- prolog:set_system_property(Var,Val), !.
+
+% unset an environment variable:
+unsetEnv(Var) :- setEnv(Var,''), !.
+
 % no warnings for singleton variables:
 noSingletonWarnings :- prolog_flag(single_var_warnings,_,off).
 
@@ -477,7 +483,8 @@ shellCmd(Cmd,Status) :-
 	%write(user_error,Cmd), nl(user_error),
 	(sicstus4
          -> absolute_file_name(path(sh),SH,[access([exist,executable])]),
-	    process_create(SH,['-c',Cmd],[process(Pid)]),
+	    catch(findall(Var=Value,environ(Var,Value),Env),_,Env=[]),
+	    process_create(SH,['-c',Cmd],[environment(Env),process(Pid)]),
             process_wait(Pid,exit(Status))
           ; system(Cmd,Status)).
 
@@ -916,8 +923,8 @@ writeqWithVars(T) :- \+ \+ (numbervars(T,0,_), writeq(T)).
 
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-% Write a block declaration for predicate PredName of arity PredArity
-% where the positions in the non-empty(!) list GroundPositions must be instantiated.
+% Write a block declaration for predicate PredName of arity PredArity where
+% the positions in the non-empty(!) list BoundPositions must be instantiated.
 % Furthermore, the last argument is a possibly new predicate name corresponding
 % to PredName which should be coded instead of PredName (this depends on
 % the implementation scheme for block declarations).
