@@ -3,51 +3,43 @@
 
 CURRYHOME=`pwd`/..
 CURRYBIN=$CURRYHOME/bin
+CURRYCHECK=`which curry-check`
+
+if [ ! -x "$CURRYCHECK" ] ; then
+  echo "Executable 'curry-check' is not installed! Skipping tests..."
+  exit 0
+fi
 
 # test for basic language features
 test_lang()
 {
-  cd LanguageTests && $CURRYBIN/curry check Test*.curry && cd ..
+  cd LanguageTests && $CURRYCHECK Test*.curry && cd ..
 }
 
 # tests for type classes:
 test_classes()
 {
-  cd TypeclassTests && $CURRYBIN/curry check Test*.curry && cd ..
-}
-
-# test for standard libraries:
-test_libs()
-{
-  cd LibraryTests && $CURRYBIN/curry check Test*.curry && cd ..
+  cd TypeclassTests && $CURRYCHECK Test*.curry && cd ..
 }
 
+# test features of specific Curry systems:
 if [ -x "$CURRYBIN/pakcs" ] ; then
-    BACKEND=`$CURRYBIN/curry :set v0 :set -time :load Distribution :eval "putStrLn (curryRuntime ++ show curryRuntimeMajorVersion)" :quit 2> /dev/null`
-    # additional library tests for PAKCS with various Prolog back ends:
-    TESTPAKCSBACKEND=
-    case "$BACKEND" in
-        sicstus3 ) TESTPAKCSBACKEND="TestIO " ;;
-        sicstus4 ) TESTPAKCSBACKEND="" ;;
-        swi5     ) TESTPAKCSBACKEND="TestIO " ;;
-    esac
-    TESTPAKCS="$TESTPAKCSBACKEND"
+    TESTPAKCS=
 elif [ -x "$CURRYBIN/kics2" ] ; then
     TESTKICS2="TestPolySubExp TestUnification"
 fi
 
-# test features of specific Curry systems:
 test_systems()
 {
   if [ -n "$TESTPAKCS" -o -n "$TESTKICS2" ] ; then
-    cd SpecialTests && $CURRYBIN/curry check $TESTPAKCS $TESTKICS2 && cd ..
+    cd SpecialTests && $CURRYCHECK $TESTPAKCS $TESTKICS2 && cd ..
   fi
 }
 
 # run all tests:
 exec_all_tests()
 {
-  test_lang && test_classes && test_libs && test_systems
+  test_lang && test_classes && test_systems
 }
 
 VERBOSE=no
